/* control structure */

/*#ifndef __ASSEMBLER__*/

/* H UART regs on AMBA_VEX */
struct uartregs
{
	volatile unsigned char uartdata1;
	unsigned char pad0;
	unsigned char pad1;
	unsigned char pad2;
	volatile unsigned char uartstatus1;
	unsigned char pad3;
	unsigned char pad4;
	unsigned char pad5;
	volatile unsigned char uartctrl1;
	//volatile unsigned int uartscaler1;
};
#define UART_BASE_ADDRESS 		0xD1000000
#define CONFIG_SYS_CLK_FREQ		75000000	/* 75MHz */
#define CONFIG_BAUDRATE			115200
#define CONFIG_SYS_GRLIB_APBUART_SCALER  ((((CONFIG_SYS_CLK_FREQ*10)/(CONFIG_BAUDRATE*8))-5)/10)



/* H hardcoded timer registers on AMBA_VEX */
struct timerregs
{
	volatile unsigned int scalercnt;
	volatile unsigned int scalerload;
	volatile unsigned int control;
	volatile unsigned int unused;
	volatile unsigned int timercnt1;
	volatile unsigned int timerload1;
	volatile unsigned int timerctrl1;
	volatile unsigned int unused2;
	volatile unsigned int timercnt2;
	volatile unsigned int timerload2;
	volatile unsigned int timerctrl2;

};
#define TIMER_BASE_ADDRESS 	 0x80000300


struct irqregs
{
	volatile unsigned int irqlevel;
	volatile unsigned int irqpend;
	volatile unsigned int irqforce;
	volatile unsigned int irqclear;
	volatile unsigned int pad[12];
	volatile unsigned int irqmask;

};
#define IRQCTRL_BASE_ADDRESS	0x80000200


struct lregs {
	volatile unsigned int memcfg1;		/* 0x00 */
	volatile unsigned int memcfg2;
	volatile unsigned int ectrl;
	volatile unsigned int failaddr;
	volatile unsigned int memstatus;		/* 0x10 */
	volatile unsigned int cachectrl;
	volatile unsigned int powerdown;
	volatile unsigned int writeprot1;
	volatile unsigned int writeprot2;	/* 0x20 */
	volatile unsigned int leonconf;
	volatile unsigned int dummy2;
	volatile unsigned int dummy3;
	volatile unsigned int dummy4;		/* 0x30 */
	volatile unsigned int dummy5;
	volatile unsigned int dummy6;
	volatile unsigned int dummy7;
	volatile unsigned int timercnt1;		/* 0x40 */
	volatile unsigned int timerload1;
	volatile unsigned int timerctrl1;
	volatile unsigned int wdog;
	volatile unsigned int timercnt2;
	volatile unsigned int timerload2;
	volatile unsigned int timerctrl2;
	volatile unsigned int dummy8;
	volatile unsigned int scalercnt;
	volatile unsigned int scalerload;
	volatile unsigned int dummy9;
	volatile unsigned int dummy10;
	volatile unsigned int uartdata1;
	volatile unsigned int uartstatus1;
	volatile unsigned int uartctrl1;
	volatile unsigned int uartscaler1;
	volatile unsigned int uartdata2;
	volatile unsigned int uartstatus2;
	volatile unsigned int uartctrl2;
	volatile unsigned int uartscaler2;
	volatile unsigned int irqmask;
	volatile unsigned int irqpend;
	volatile unsigned int irqforce;
	volatile unsigned int irqclear;
	volatile unsigned int piodata;
	volatile unsigned int piodir;
	volatile unsigned int pioirq;
};


/*#endif*/

/* control registers */

#define	PREGS	0x80000000
#define	MCFG1	0x00
#define	MCFG2	0x04
#define	ECTRL	0x08
#define	FADDR	0x0c
#define	MSTAT	0x10
#define CCTRL	0x14
#define PWDOWN	0x18
#define WPROT1	0x1C
#define WPROT2	0x20
#define LCONF 	0x24
#define	TCNT0	0x40
#define	TRLD0	0x44
#define	TCTRL0	0x48
#define	TCNT1	0x50
#define	TRLD1	0x54
#define	TCTRL1	0x58
#define	SCNT  	0x60
#define	SRLD  	0x64
#define	UDATA0 	0x70
#define	USTAT0 	0x74
#define	UCTRL0 	0x78
#define	USCAL0 	0x7c
#define	UDATA1 	0x80
#define	USTAT1 	0x84
#define	UCTRL1 	0x88
#define	USCAL1 	0x8c
#define	IMASK	0x90
#define	IPEND	0x94
#define	IFORCE	0x98
#define	ICLEAR	0x9c
#define	IOREG	0xA0
#define	IODIR	0xA4
#define	IOICONF	0xA8

/* ASI codes */

#define ASI_PCI 	0x4
#define ASI_ITAG	0xC
#define ASI_IDATA	0xD
#define ASI_DTAG	0xE
#define ASI_DDATA	0xF

/* memory areas */

#define CRAM	0x40000000
#define IOAREA	0x20000000

/* Some bit field masks */

#define CCTRL_FLUSHING_MASK 0x0c000

#define CPP_CONF_BIT	28
#define CPP_CONF_MASK	3
#define FPU_CONF_BIT	4
#define FPU_CONF_MASK	3
#define CPTE_MASK	0x0c0000
#define ITE_BIT		16
#define IDE_BIT		14
#define DTE_BIT		12
#define DDE_BIT		10

/* I don't think we have all of these. What are they for? */
#define USTAT_PE 0
#define USTAT_FE 0
#define UCTRL_PE 0
#define UCTRL_PS 0
#define UCTRL_FL 0

#define USTAT_TXDE  0x01  /* TX data register empty. */
#define UCTRL_TXDEE 0x01  /* TX data register empty interupt enable. */
#define USTAT_TXDR  0x02  /* TX data register ready. */
#define UCTRL_TXDRE 0x02  /* TX data register ready interupt enable. */
#define USTAT_TOV   0x04  /* Transmit overflow. */
#define USTAT_RXDR  0x08  /* RG data register ready. */
#define UCTRL_RXDRE 0x08  /* RG data register ready interupt enable. */
#define USTAT_CTI   0x10  /* Character timeout interrupt. */
#define USTAT_CTIE  0x10  /* Character timeout interrupt enable. */
#define USTAT_ROV   0x20  /* Receive overflow. */

